/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowsePageCondition;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;

/**
 * Task relations DAO.
 * 
 * @author yosuke
 * 
 */
@S2Dao(bean = TaskRelation.class)
public interface TaskRelationsDao {
    /**
     * find by task Id.
     * 
     * @param id
     * @param userId
     * @return
     */
    @Arguments( { "id", "userId" })
    TaskRelation findByTaskId(int id, int userId);

    /**
     * find by user Id.
     * 
     * @param userId
     * @return
     */
    @Arguments( { "userId", "isIncludeDone" })
    TaskRelation[] findByUserId(int userId, boolean isIncludeDone);

    /**
     * get relations with histories by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    TaskRelation[] getRelationsWithHistoriesByUserId(int userId);

    
    /**
     * find task ids by query params.
     * 
     * @param condition
     * @return
     */
    @Arguments("condition")
    TaskRelation[] findTaskIdsByQueryParams(TaskBrowsePageCondition condition);
    
    /**
     * find by query params.
     * 
     * @param condition
     * @return
     */
    @Arguments( { "taskIds", "userId" })
    TaskRelation[] findByQueryParams(Integer[] taskIds, int userId);

    /**
     * get note rows by query params.
     * 
     * @param condition
     * @return
     */
    @Arguments("condition")
    TaskRelation getRowsByQueryParams(TaskBrowsePageCondition condition);

    /**
     * get default foloder tasks.
     * 
     * @param activities
     * @return
     */
    @Arguments( { "userId", "activities" })
    TaskRelation getDefaultFolderTasks(int userId, int[] activities);

    /**
     * get person in charge by activity id.
     * 
     * @param userId
     * @param activityId
     * @return
     */
    @Arguments("activityIds")
    TaskRelation[] getPersonInChargeByActivityIds(Integer[] activityIds);

    /**
     * get person in charge by own activities.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    TaskRelation[] getPersonInChargeByOwnActivities(int userId);

    /**
     * get count by activity id.
     * 
     * @param activityId
     * @return
     */
    @Arguments("activityId")
    int getCountByActivityId(int activityId);

    /**
     * get count by activity id and person id.
     * 
     * @param activityId
     * @param personId
     * @return
     */
    @Arguments( { "activityId", "personId" })
    int getCountByActivityIdAndPersonId(int activityId, int personId);

    /**
     * insert task relations.
     * 
     * @param task
     * @return
     */
    @Arguments("taskRelations")
    int insert(TaskRelation taskRelations);

    /**
     * update task relations.
     * 
     * @param task
     * @return
     */
    @Arguments("taskRelations")
    int update(TaskRelation taskRelations);

    /**
     * update classification.
     * 
     * @param taskId
     * @param classId
     * @param statusId
     * @return
     */
    @Arguments( { "taskId", "classId", "statusId" })
    int updateClassification(int taskId, int classId, int statusId);

    /**
     * update classifications for transfer tasks.
     * 
     * @param taskIds
     * @return
     */
    @Arguments("taskIds")
    int updateClassificationsForTransferTasks(Integer[] taskIds);

    /**
     * get update target by date for move to archive.
     * 
     * @param date
     * @return
     */
    @Arguments("date")
    TaskRelation[] getUpdateTargetByDateForMoveToArchive(String date);

    /**
     * update classification move to archive.
     * 
     * @param taskIds
     * @return
     */
    @Arguments("taskIds")
    int updateClassificationsMoveToArchive(Integer[] taskIds);

    /**
     * find task by discussion id.
     * 
     * @param toTask
     * @return
     */
    @Arguments("discussionId")
    TaskRelation findTaskByDiscussionId(int discussionId);

    /**
     * find task ids by has tags.
     * 
     * @param condition
     * @return
     */
    @Arguments("condition")
    TaskRelation[] findTaskIdsByHasTags(TaskBrowsePageCondition condition);
    
    /**
     * find by has tags.
     * 
     * @param condition
     * @return
     */
    @Arguments( { "taskIds", "userId" })
    TaskRelation[] findByHasTags(Integer[] taskIds, int userId);

    /**
     * get rows by has tags.
     * 
     * @param condition
     * @return
     */
    @Arguments("condition")
    int getRowsByHasTags(TaskBrowsePageCondition condition);
    
    /**
     * find by class id.
     * 
     * @param classId
     * @param userId
     * @return
     */
    @Arguments({"classId","userId"})
    TaskRelation[] findByClassId(int classId, int userId);
    
    /**
     * get due list by user id/
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    TaskRelation[] getDueListByUserId(int userId);
    
    /**
     * find by deadline
     * 
     * @param deadline
     * @param userId
     * @return
     */
    @Arguments({"deadline","userId"})
    TaskRelation[] findByDeadline(String deadline, int userId);
    
}